# SPDX-FileCopyrightText: 2022 Gregory Clunies <greg@reflekt-ci.com>
#
# SPDX-License-Identifier: Apache-2.0

# flake8: noqa
# fmt: off
dbt_src_schema = {
    "version": 2,
    "sources": [
        {
            "name": "",
            "schema": "",
            "database": "",
            "description": "",
            "tables": []
        }
    ]
}

dbt_table_schema = {
    "name": "",
    "description": "",
    # "columns": [],
}

dbt_doc_schema = {
    "version": 2,
    "models": [],
}

dbt_model_schema = {
    "name": "",
    "description": "",
    "columns": [],
}

dbt_column_schema = {
    "name": "",
    "description": "",
}
# fmt: on
