# SPDX-FileCopyrightText: 2022 Gregory Clunies <greg@reflekt-ci.com>
#
# SPDX-License-Identifier: Apache-2.0

TITLE_CASE_RE = r"[A-Z][a-z]+"
CAMEL_CASE_RE = r"(^[a-z]|[A-Z])[a-z]*"
SNAKE_CASE_RE = r"^[a-z._]+$"
