# SPDX-FileCopyrightText: 2022 Gregory Clunies <greg@reflekt-ci.com>
#
# SPDX-License-Identifier: Apache-2.0
class ReflektValidationError(Exception):
    pass


class ReflektProjectError(Exception):
    pass


class ReflektConfigError(Exception):
    pass
